#!perl
#
# Master of Epic location to Twitter
# by Sharl Morlaroll <sharl@hauN.org>

use strict;
use warnings;
use Config::IniFiles;
use Net::Twitter;
use Jcode;

my $DEBUG = 1;

##############
# initialize #
##############
my %ini = get_setting();
my %Servers = get_servers($ini{General}{Servers});
my $tw = Net::Twitter->new(
			   username => $ini{Twitter}{Username},
			   password => $ini{Twitter}{Password}
			   );

# main
my ($login, $appear, $pre_loc) = (0, 0, "");
while (1) {
    my $game = 0;
    my @foriegns = get_foriegns();

    foreach my $f (@foriegns) {
	my $loc = $Servers{$f};
	$game = 1 if (defined($loc) && "$loc" eq "game");
	if (! $login && $game) {
	    defined($tw) && $tw->update(jcode("$ini{Words}{Login}")->utf8);
	    print "$ini{Words}{Login}\n";
	    $login = 1;
	}

	if ($game && $loc ne $pre_loc && $loc ne "login" && $loc ne "game") {
	    if (!$appear) {
		defined($tw) && $tw->update(jcode("$loc$ini{Words}{Appear}")->utf8);
		print "$loc$ini{Words}{Appear}\n";
		$appear = 1;
	    } else {
		defined($tw) && $tw->update(jcode("$loc$ini{Words}{Move}")->utf8);
		print "$loc$ini{Words}{Move}\n";
	    }
	    $pre_loc = $loc;
	}

	if ($DEBUG && ! defined $loc) {
	    my ($s, $p) = split(/:/, $f);
	    if ($s =~ /^202\.12\./) {
		print "undefined $f\n";
	    }
	}
    }
    if ($login && !$game) {
	defined($tw) && $tw->update(jcode("$ini{Words}{Logout}")->utf8);
	$DEBUG && print "$ini{Words}{Logout}\n";
	($login, $appear, $pre_loc) = (0, 0, "");
    }
    sleep 5;
}

############################################################
sub get_setting {
    my %ini;
    tie %ini, 'Config::IniFiles', (-file => "setting.ini");

    return %ini;
}
############################################################
sub get_servers {
    my %servers;
    my $fd;
    open($fd, $_[0]) || die $!;
    while (<$fd>) {
	/^$/ && next;
	/^#/ && next;
	chomp;

	my ($server_port, $age, $loc) = split(/\//);
	$servers{$server_port} = $loc;
    }
    close($fd);

    return %servers;
}
############################################################
sub get_foriegns {
    my @foriegns = ();

    my $fd;
    open($fd, "netstat |");
    my @buffer = <$fd>;
    close($fd);

    foreach my $line (@buffer) {
	if ($line =~ /TCP/ && $line =~ /ESTABLISHED/ && $line =~ /202\.12\.243\./) {
	    my (undef, $prot, $local, $foriegn, $state) = split(/\s+/, $line);
	    push @foriegns, $foriegn;
	}
    }
    return @foriegns;
}
